package Dir;

use strict;

# -----------------------------------------------------------------------------

sub new() {

    my $class = shift;
    my $self  = { };
    my $args  = { # ...no defaults...
                  # ... => ...,
                  @_ };

    bless($self, $class);

    # -- check that things which must be defined have been :
    unless (defined $args->{DIR}) {
        die "\n\n Gotta spec DIR!!! \n\n";
      }
    unless (defined $args->{TYPE}) {
        die "\n\n Gotta spec TYPE!!! \n\n";
      }

    $self->{DIR}           = $args->{DIR};
    $self->{SUB_DIRS_LIST} = [ ];

    # -- recursively go down given dir/tree :

    push(@{$self->{SUB_DIRS_LIST}}, 
         @{$self->get_dir($args->{DIR}, $args->{TYPE})});

    return $self;
  }


# -----------------------------------------------------------------------------
# -- CONSTRUCTOR SLAVES :
# -----------------------------------------------------------------------------

sub get_dir() {

    my $self = shift;
    my $dir  = shift;
    my $type = shift;

    my $sub_dirs = [ $dir ];

    opendir(DIR, $dir)
        || die "\n\n Can't open dir:  $dir \n\n";

    foreach my $name (sort readdir(DIR)) {
        if (($name eq ".") || ($name eq "..")) {
          }
        elsif (-d "$dir/$name") {
            push(@$sub_dirs, "$dir/$name");
            if ($type eq "RECURSIVE") {
                push(@$sub_dirs, @{$self->get_dir("$dir/$name")});
              }
            elsif ($type eq "SIMPLE") {
              }
            else {
                die "\n\n File type = $type ???  " 
                  . "\n (Should be \"SIMPLE\" or \"RECURSIVE\")\n\n";
              }
          }
        else {
            # ...not a directory so ignore...
          }
      }

    close(DIR);

    return $sub_dirs;
  }


# -----------------------------------------------------------------------------
# -- READ METHODS :
# -----------------------------------------------------------------------------

# -- return a pointer to list of sub-dirs :

sub sub_dirs_list() {
    my $self = shift;
    return $self->{SUB_DIRS_LIST};
  }


# -----------------------------------------------------------------------------
# -- return pointer to list of pointers to hashes representing 
#    ordinary (e.g, non-directory) files in given directory, where hashes 
#    include std file attributes (see &get_file_stats) :

sub non_dir_files() {

  my $self = shift;
  my $dir  = shift;

    my @nd_files = ();

    opendir(DIR, $dir) 
        || die "\n\n Can't open dir: " . $dir . "\n\n";

    foreach my $name (sort readdir(DIR)) {
        if (($name eq ".") || ($name eq "..")) {
          }
        elsif (-f $dir . "/$name") {
            my $file = $self->get_file_stats($dir, $name);
            push(@nd_files, $file);
          }
        else {
          }
      }

    closedir(DIR);

    return \@nd_files;
  }


# -----------------------------------------------------------------------------
# -- return pointer to list of pointers to hashes representing 
#    files as returned by `ls -al` (excluding . and ..) in given directory, 
#    where hashes include std file attributes (see &get_file_stats) :

sub ls_al() {
    my $self = shift;
    my $dir  = shift;

    return "this is the list";
  }


# -----------------------------------------------------------------------------
# -- 

sub ls_alR() {
    my $self = shift;

    my $ls_alR = `ls -alR $self->{DIR}`;

    return \$ls_alR;
  }


# -----------------------------------------------------------------------------
# -- TOOL KIT :
# -----------------------------------------------------------------------------

sub get_file_stats() {
    my $self     = shift;
    my $dir      = shift;
    my $filename = shift;
    my $file     = { };    

            $file->{NAME}     = $filename;
            $file->{FULLNAME} = $dir . "/$filename";

    (undef,          $file->{INODE}, $file->{MODE},  undef, 
     $file->{UID},   $file->{GID},   undef,          $file->{SIZE},
     undef,          $file->{MTIME}, $file->{CTIME}, undef,
     $file->{BLOCKS}) 
                = stat($file->{FULLNAME});
   
    return $file;
  }


# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------
