# -----------------------------------------------------------------------------
package Dir_MD5;

# -- sub-class of Dir:  this adds MD5 info.

# -----------------------------------------------------------------------------


use strict;
use vars qw(@ISA);
use Dir;
use Digest::MD5;

@ISA = ("Dir");


sub new() {
    my $class = shift;
    my $self  = { };
    my $args  = { # ...no defaults...
                  # ... => ...,
                  @_ };

    bless($self, $class);

    # -- check that things which must be defined have been :
    unless (defined $args->{DIR}) {
        die "\n\n Gotta spec DIR!!! \n\n";
      }

    unless (defined $args->{TYPE}) {
        die "\n\n Gotta spec TYPE!!! \n\n";
      }

    $self->{DIR}           = $args->{DIR};
    $self->{SUB_DIRS_LIST} = [ ];

    # -- recursively go down given dir/tree :

    push(@{$self->{SUB_DIRS_LIST}}, 
         @{$self->get_dir($args->{DIR}, $args->{TYPE})});

    return $self;
  }


# -----------------------------------------------------------------------------
# -- READ METHODS :
# -----------------------------------------------------------------------------

# -- return md5 for given file :

sub md5() {
    my $self     = shift;
    my $fullname = shift;

    unless ($fullname =~ m/$self->{DIR}/) {
        die "\n\n Dir_MD5::chk_str :  " 
          . "file ($fullname) not in $self->{DIR}. \n\n";
      }

    my $file = { };

    (undef,          $file->{INODE}, undef,          undef, 
     undef,          undef,          undef,          undef,
     undef,          $file->{MTIME}, undef,          undef,
     undef) 
                = stat($fullname);

    open(F, $fullname) 
        || die "\n\n Dir_MD5::md5 :  Can't open file : $fullname \n\n";
    $file->{MD5} = Digest::MD5->new->addfile(*F)->hexdigest();
    close(F);
    
    return $file->{MD5};
  }


# -- return check-string for given file :

sub file_chk_str() {
    my $self     = shift;
    my $fullname = shift;

    unless ($fullname =~ m/$self->{DIR}/) {
        die "\n\n Dir_MD5::chk_str :  " 
          . "file ($fullname) not in $self->{DIR}. \n\n";
      }

    my $file = { };

    (undef,          $file->{INODE}, undef,          undef, 
     undef,          undef,          undef,          undef,
     undef,          $file->{MTIME}, undef,          undef,
     undef) 
                = stat($fullname);

    open(F, $fullname) 
        || die "\n\n Can't open file : $fullname \n\n";
    $file->{MD5} = Digest::MD5->new->addfile(*F)->hexdigest();
    close(F);
    
    return "$file->{MTIME} $file->{INODE} $file->{MD5}";
  }


# -----------------------------------------------------------------------------
# -- as for Dir::non_dir_files, but MD5 checksums added :

sub non_dir_files() {

  my $self = shift;
  my $dir  = shift;

    my $nd_files = $self->Dir::non_dir_files($dir);

    foreach (@$nd_files) {
        open(F, $_->{FULLNAME}) 
            || die "\n\n Dir_MD5::non_dir_files :  " 
                 . "Can't open file : $_->{FULLNAME} \n\n";
        $_->{MD5} = Digest::MD5->new->addfile(*F)->hexdigest();
        close(F);
      }

    return $nd_files;
  }


# -----------------------------------------------------------------------------
# -- as for Dir::get_file_stats, but MD5 checksum added :

sub get_file_stats() {
    my $self     = shift;
    my $dir      = shift;
    my $filename = shift;

    my $file = self->Dir::get_file_stats($dir, $filename);

    $file->{NAME}     = $filename;
    $file->{FULLNAME} = $dir . "/$filename";

    open(F, $file->{FULLNAME}) 
        || die "\n\n Dir_MD5::get_file_stats :  " 
             . "Can't open file : $file->{FULLNAME} \n\n";
    $file->{MD5} = Digest::MD5->new->addfile(*F)->hexdigest();
    close(F);
   
    return $file;
  }

# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------
