# -----------------------------------------------------------------------------
package Log;
# -----------------------------------------------------------------------------

use strict;
use File::Copy;
use Sys::Syslog;


##my $week = 60*60*24*7;
my $week = 60*60*24;
    # ...a week in seconds;


sub new() {
    my $class = shift;
    my $self  = { MODE => "std",
                  SUFFIX => "log",
                  BUFFERED => "yes",
                  SYSLOG   => "yes",
                  @_ };

    unless ($self->{MODE} =~ m/std|showprocnames/ ) {
        die "\n\n  -- Unknown MODE in Log::new -- \n\n";
      }

    unless ($self->{BUFFERED} =~ m/yes|no/ ) {
        die "\n\n  -- Unknown BUFFERED in Log::new -- \n\n";
      }

    unless ($self->{SYSLOG} =~ m/yes|no/ ) {
        die "\n\n  -- Unknown SYSLOG in Log::new -- \n\n";
      }

    unless (defined $self->{MODULE}) {
        die "\n\n  -- Unknown MODULE in Log::new -- \n\n";
      }

    unless (defined $self->{PREFIX}) {
        die "\n\n  -- Unknown PREFIX in Log::new -- \n\n";
      }

    unless (defined $self->{ROOT_DIR}) {
        die "\n\n  -- Unknown ROOTDIR in Log::new -- \n\n";
      }

    $self->{NAME}         = $self->{PREFIX} . "." . $self->{SUFFIX};
    $self->{FULLNAME}     = $self->{ROOT_DIR} . "/var/log/" . $self->{NAME};
    $self->{FULLNAMEWARN} = $self->{ROOT_DIR} . "/var/log/WARNINGS_" . $self->{NAME};
    $self->{LASTTIME}     = time;
        # ...seconds since last rotation; 

    bless($self, $class);    

    $self->open_OUT();

    return $self;
  }


sub wen() {
    my $self= shift;

    $self->__close_OUT();
  }


# -----------------------------------------------------------------------------
# --
# -----------------------------------------------------------------------------

sub output_title() {
    my $self  = shift;
    my $title = shift;

    $self->append("");
    $self->append("");
    $self->append("-- $title :: ------------------------------------------------------------");
  }


sub output_hash() {
    my $self   = shift;
    my $title  = shift;
    my $hash   = shift;
    my $format = shift;

    $self->append("");
    $self->append($title . " : ");
    $self->indent();
    
    foreach my $k (sort keys %$hash) {
        my $string      = 'sprintf("' . $format->[0] . '", "' . $k 
                  . '") . ":" .  sprintf("' . $format->[1] . '", "' . $hash->{$k} . '")';
        my $eval_string = eval($string);

        $self->append($eval_string);
      }

    $self->un_indent();
    $self->append(" ..." . $title . ".");
    $self->append("");
  }


sub output_array() {
    my $self   = shift;
    my $title  = shift;
    my $array  = shift;
    my $format = shift;

    $self->append("");
    $self->append($title . " : ");
    $self->indent();

    my $string = '""';

    while (my $f = shift @$format) {
        my $g = shift @$format;
        $string .=  ' . sprintf("' . $f . '", "' . $g . '")';
      }

    foreach my $c (@$array) {
        my $eval_string = eval($string);
        $self->append($eval_string);
      }

    $self->un_indent();
    $self->append(" ..." . $title . ".");
    $self->append("");
  }


# -----------------------------------------------------------------------------
# -- 
# -----------------------------------------------------------------------------

sub append_echo_warn() {
    my $self     = shift;
    my $string   = shift;
    my $warn_str = "*** WARNING: " . $self->{INDENT_STR} . $string;

    print "\n\n $warn_str \n";

    $self->warn($string);
    $self->append("*** WARNING: " . $string);
  }


sub append_echo() {
    my $self   = shift;
    my $string = shift;

    print "\n " . $self->{INDENT_STR} . $string;

    $self->append($string);
  }


sub warn() {
    my $self     = shift;
    my $string   = shift;
    my $warn_str = "*** WARNING: " . $self->{INDENT_STR} . $string;
    my $warn_out = $self->{FULLNAMEWARN};

    my ($seconds, $minutes, $hours,  
        $day_o_m, $month, $year, undef, undef, undef) = localtime(time);

    $year  += 1900;
    $month += 1;

    `echo  "$year/$month/$day_o_m $hours:$minutes:$seconds  $warn_str" >> $warn_out`;
  }


sub append() {
    my $self   = shift;
    my $string = shift;
    unless ($string =~ m/./) {$string = " "};

    print {$self->{OUT}} "\n " . $self->{INDENT_STR} . $string;

    if ($self->{SYSLOG} eq "yes") {
        openlog("sids", "ndelay", "local7");
        syslog("notice", "SIDS: " . $string);
        closelog(); 
      }

    if (time > $self->{LASTTIME} + $self->{MODSCFG}->module_rot_prd($self->{MODULE})) {
        $self->{LASTTIME} = time;
        $self->rotate_log();
      }

  }


sub append_f() {
    my $self= shift;

    die "\n\n Log::append_f :  NYI \n\n";
  }

sub flush() {
    my $self= shift;

    die "\n\n Log::append_f :  NYI \n\n";
  }

sub indent() {
    my $self= shift;
    $self->{INDENT}++;
    $self->{INDENT_STR} .= "    ";
  }

sub un_indent() {
    my $self= shift;
    $self->{INDENT}--;
    $self->{INDENT_STR} =~ s/    //;
  }


# -----------------------------------------------------------------------------
# -- TOOLKIT :
# -----------------------------------------------------------------------------

sub open_OUT() {
    my    $self = shift;

    $self->rotate_log();
    $self->__open_OUT();
  }

sub __close_OUT() {
    my $self = shift;

    close $self->{OUT};
  }


sub __open_OUT() {
    my    $self = shift;
    local *OUT;   # ++ see Cookbook, P257 (near top of page)

    open (OUT, ">" . $self->{FULLNAME}) 
        || die("\n\n ** Can't open app-stream: " . $self->{NAME} . " ** \n\n");

    if ($self->{BUFFERED} eq "no") {
        select(OUT);  $| = 1;  select(STDOUT);
      }

    $self->{OUT}    = *OUT;
    $self->{INDENT} = 0;

    if ((defined $self->{OWNER_UID}) && (defined $self->{OWNER_GID})) {
        chown($self->{OWNER_UID}, $self->{OWNER_GID}, ,$self->{FULLNAME});
      }
  }


sub rotate_log() {
    my $self = shift;
    my $i    = 0;
    my $NUM  = 7;
    my $fnw  = $self->{FULLNAMEWARN};

    $self->__close_OUT();

    for ($i = 0;  $i < $NUM; $i++) {
        my $ni_1 = $NUM - $i - 1;
        my $ni   = $NUM - $i;

        print"\n mv $self->{FULLNAME}.$ni_1 $self->{FULLNAME}.$ni... ";
        move("$self->{FULLNAME}.$ni_1", "$self->{FULLNAME}.$ni");

        print"\n mv $self->{FULLNAMEWARN}.$ni_1 $self->{FULLNAMEWARN}.$ni... ";
        move("$self->{FULLNAMEWARN}.$ni_1", "$self->{FULLNAMEWARN}.$ni");
      }

    print"\n mv $self->{FULLNAME} $self->{FULLNAME}.0... ";
    move("$self->{FULLNAME}", "$self->{FULLNAME}.0");

    print "\n open $self->{FULLNAME}...";
    $self->__open_OUT();

    print"\n mv $self->{FULLNAMEWARN} $self->{FULLNAMEWARN}.0... ";
    move("$self->{FULLNAMEWARN}", "$self->{FULLNAMEWARN}.0");

    `touch $fnw`;
        # ...create new WARNINGS file...
  }

# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------

