# -----------------------------------------------------------------------------
package IDM_Sigs;
# -----------------------------------------------------------------------------

use strict;


# -----------------------------------------------------------------------------
# -- IDM_procs
# -----------------------------------------------------------------------------


sub _get_procs_lsof_sigs() {
    my $self = shift;
    my $mode = shift;
    my $sigs = shift;

    my $root = $self->{SIDSCONFIG}->root_dir() . "/etc/IDM_procs/expected";
    my $file = $root . "." . lc($mode);


    open(CFG, $file) || 
        die "\n\n IDM_procs::_get_procs_lsof_sigs :  Can't open file: $file \n\n";

    my $ps_name = "";
    my $BLOCK   = "";

    while(my $line = <CFG>) {
        chomp($line);

        if     ($line =~ m/^\s*\#/) {next}
        unless ($line =~ m/\S/)     {next}

        if ($line =~ m/^PS\:/){
            $line =~ s/^PS\:\s*//;
            $line =~ s/\s$//;
            $ps_name = $line;
            $sigs->{$ps_name} = { };
            undef $BLOCK;
	  }

        elsif ($line =~ m/\s+CWD|AOUT|OFILES|ODEVS|INET\s+\=\s*/) {
            $BLOCK = $line;
            $BLOCK =~ s/\s+([A-Z]+)\s+\=\s*/$1/;
            $sigs->{$ps_name}->{$BLOCK} = [ ];
#if ($line =~ m/AOUT/) {
#    print "\n line AOUT $ps_name";
#  }
          }

        elsif ($line =~ m/\s+QFILES|QDEVS\s+\=\s*/) {
            $BLOCK = $line;
            $BLOCK =~ s/\s+([A-Z]+)\s+\=\s*/$1/;
            $sigs->{$ps_name}->{$BLOCK} = { };
          }

        elsif ($line =~ m/^\s*\"[^\"]+\"\s*$/) {
            $line =~ s/^\s*//g;
            $line =~ s/\s*$//g;
            $line =~ s/\"//g;         

            if (($BLOCK eq "QFILES") || ($BLOCK eq "QDEVS")) {
                $sigs->{$ps_name}->{$BLOCK}->{$line} = 1;
              }
            elsif (($BLOCK eq "OFILES") || ($BLOCK eq "ODEVS")) {
                push(@{$sigs->{$ps_name}->{$BLOCK}}, $line);
              }
            elsif ($BLOCK eq "CWD") {
                push(@{$sigs->{$ps_name}->{CWD}}, $line);
	      }
            elsif ($BLOCK eq "AOUT") {
                push(@{$sigs->{$ps_name}->{AOUT}}, $line);
print "\n block aout $ps_name " . $sigs->{$ps_name}->{AOUT};
                # -- use AOUT value as ps-name (which is definitely unique) :
                $sigs->{$line} = $sigs->{$ps_name};
		delete($sigs->{$ps_name});
                $ps_name = $line;
	      }
            elsif ($BLOCK eq "INET") {
                # ...not done yet...
	      }
            else {
                die "\n\n Unknown block: $BLOCK \n\n";
              }
          }
        else {
            die "\n\n Unknown line: $line \n\n";
          }
      }
    close(CFG);
  }




# -----------------------------------------------------------------------------

sub _get_procs_api_sigs() {
    my $self = shift;
    my $mode = shift;
    my $sigs = shift;

    my $root = $self->{SIDSCONFIG}->root_dir() . "/etc/IDM_procs/expected";
    my $file = $root . "." . lc($mode);

    my @cmds_eq  = ();
    my @cmds_ma  = ();

        open(SIGS, $file) 
            || die "\n\n IDM_procs::_get_procs_api_sigs :  " 
                 . "cannot open file: $file//\n\n";

        while(my $line = <SIGS>) {
            chomp($line);

            unless ($line =~ m/\S/){next;}
            if ($line =~ m/^\s*\#/){next;}
           
            if ($line =~ m/^EQ\s\"[A-Za-z0-9\/\-\_\ \.\:\[\]\=\,]+\"\s*$/) {
                $line =~ s/EQ\s\"([A-Za-z0-9\/\-\_\ \.\:\[\]\=\,]+)\"/$1/;
                push(@cmds_eq, $line);
              }
            elsif ($line =~ m/^MA\s\"[A-Za-z0-9\/\-\_\ \.\:\\\+\[\]@]+\"\s*$/) {
                   $line =~ s/MA\s\"([A-Za-z0-9\/\-\_\ \.\:\\\+\[\]@]+)\"/$1/;
                push(@cmds_ma, $line);
              }
            else{
                die "\n\n IDM_procs::_get_procs_api_sigs :  " 
                  . "unknown line: $line//\n\n";
              }
          }
        close(SIGS);

    $sigs->{EQ} = \@cmds_eq;
    $sigs->{MA} = \@cmds_ma;

    return $sigs;
  }


# -----------------------------------------------------------------------------
# --


sub _matches_proc_sigs() {
    my $self = shift;
    my $proc = shift;
    my $sigs = shift;

    foreach my $s (@{$sigs->{EQ}}) {
        my $cmndline = $proc->cmndline();    $cmndline =~ s/\s*$//;
        if ($cmndline eq $s) {return 1}
      }

    foreach my $s (@{$sigs->{MA}}) {
        my $cmndline = $proc->cmndline();    $cmndline =~ s/\s*$//;
        if ($cmndline =~ m/$s/) {return 1}
      }


    return 0;
  }


# -----------------------------------------------------------------------------
# -- 

sub _matches_procs_lsof_aout() {
    my $self   = shift;
    my $lsof   = shift;
    my $pk     = shift;
    my $sig    = shift;
    my $detail = shift;
    my $cmd  = $lsof->proc_attrib(PROC=>$pk, ATTRIB=>"COMMAND");
    my $result = 0;

    if (defined $sig->{AOUT}) {

        push(@$detail, "Sig AOUT (" . $sig->{AOUT}->[0] 
                                    . ") does not match cmd: $cmd");

        foreach my $ek (keys %{$lsof->open_files(PROC=>$pk)}) {
            if ($sig->{AOUT}->[0] eq $lsof->file_attrib(PROC=>$pk, FILE=>$ek, ATTRIB=>"NAME")) {
                $result = 1;
                last;
              }
          }
      }

    # -- if there is no AOUT defined in our sig we assume all is ok (since it 
    #    must be unimportant!) :
    #
    else {  $result = 1;  }

    if ($result == 1) {  $detail->[0] = "";  }

    return $result;
  }

# -----------------------------------------------------------------------------
# -- 

sub _matches_procs_lsof_cwd() {
    my $self   = shift;
    my $lsof   = shift;
    my $pk     = shift;
    my $sig    = shift;
    my $detail = shift;
##  my $cmd    = $ps->{PS}->{COMMAND};
    my $cmd  = $lsof->proc_attrib(PROC=>$pk, ATTRIB=>"COMMAND");
    my $result = 0;

    if (defined $sig->{CWD}) {
##      foreach my $e (keys %{$ps->{FILES}}) {
        foreach my $ek (keys %{$lsof->open_files(PROC=>$pk)}) {
##        unless ($ps->{FILES}->{$e}->{DESCR} eq "cwd") {next;}
          unless ($lsof->file_attrib(PROC=>$pk, FILE=>$ek, ATTRIB=>"DESCR") eq "cwd") {next;}

            if ($self->debug_verbose()) {
                print "\n CWD SIG: $sig->{CWD}->[0]";
##              print "\n CWD NEW: $ps->{FILES}->{$e}->{NAME}";
                print "\n CWD NEW: " . $lsof->file_attrib(PROC=>$pk, FILE=>$ek, ATTRIB=>"NAME");
	      }

##          if ($sig->{CWD}->[0] eq $ps->{FILES}->{$e}->{NAME}) {
            if ($sig->{CWD}->[0] eq $lsof->file_attrib(PROC=>$pk, FILE=>$ek, ATTRIB=>"NAME")) {
                $result = 1;
              }       
            else {
                push(@$detail, "Current working dir (" 
                           . $sig->{CWD}->[0] 
                           . ") from sig does not match: $cmd " . "(" 
##                         . $ps->{FILES}->{$e}->{NAME} . ")");
                           . $lsof->file_attrib(PROC=>$pk, FILE=>$ek, ATTRIB=>"NAME") . ")");
              }
          }
      }

    # -- if there is no CWD defined in our sig we assume all is ok (since it 
    #    must be unimportant!) :
    #
    else {  $result = 1;  }

    return $result;
  }


# -----------------------------------------------------------------------------
# -- 

sub _matches_procs_lsof_ods($ps) {
    my $self   = shift;
    my $lsof   = shift;
    my $pk     = shift;
    my $sig    = shift;
    my $detail = shift;
##  my $cmd    = $ps->{PS}->{COMMAND};
    my $cmd  = $lsof->proc_attrib(PROC=>$pk, ATTRIB=>"COMMAND");
    my $result = 0;

    # -- we have an open-devices signature defined :
    if (defined $sig->{ODEVS}) {
 
        # -- compute the symmetric difference between the expected list
        #    of open devices and the list found:

        my %count = ();

        # -- increment count of device files :
##      foreach my $e (keys %{$ps->{FILES}}) {
        foreach my $ek (keys %{$lsof->open_files(PROC=>$pk)}) {
##          unless ($ps->{FILES}->{$e}->{NAME} =~ m/\/dev\//) {next;}
            unless ($lsof->file_attrib(PROC=>$pk, FILE=>$ek, ATTRIB=>"NAME") =~ m/\/dev\//) {next;}
##            $count{$ps->{FILES}->{$e}->{NAME}}++;
            $count{$lsof->file_attrib(PROC=>$pk, FILE=>$ek, ATTRIB=>"NAME")}++;
          }

        # -- increment count of expected such files :
        foreach my $e (@{$sig->{ODEVS}}) {$count{$e}++;}

        my @diff = ();
        foreach my $e (keys %count) {
            # -- if found not once then we have a difference :
            if ($count{$e} == 1) { 
                # -- unless its a "?" open file :
                unless (defined $sig->{QDEVS}->{$e}) {
                    push(@diff, $e)
                  }
              }
          }

        # -- log any found differences :
        foreach (@diff) {push(@$detail, "    open dev sig diff: $_");}
	if (scalar @diff > 0) {
            push(@$detail, "Open-devs signature does not match: $cmd"); 
	    $result = 0;
          }
        else {  $result = 1;  }
      }

    # -- there is no open-devices signature :
    else {
        $result = 1;
      }

    return $result;
  }


# -----------------------------------------------------------------------------
# --

sub _matches_procs_lsof_ofs() {
    my $self   = shift;
    my $lsof   = shift;
    my $pk     = shift;
    my $sig    = shift;
    my $detail = shift;
##  my $cmd    = $ps->{PS}->{COMMAND};
    my $cmd  = $lsof->proc_attrib(PROC=>$pk, ATTRIB=>"COMMAND");
##  my $ppid   = $ps->{PS}->{PPID};
    my $ppid   = $lsof->proc_attrib(PROC=>$pk, ATTRIB=>"PPID");
##  my $gid    = $ps->{PS}->{GID};
    my $gid    = $lsof->proc_attrib(PROC=>$pk, ATTRIB=>"GID");
    my $result = 0;


    # -- if we have a signature defined for this :
    #
    if (defined $sig->{OFILES}) {

        # -- if verbose-debugging, print sig list and actual list of open files in 
        #    full :
        if ($self->debug_verbose()) {
            print "\n open files: ";
##          foreach my $op_f (keys %{$ps->{FILES}}) {
            foreach my $op_f (keys %{$lsof->open_files(PROC=>$pk)}) {
##              unless ($ps->{FILES}->{$op_f}->{TYPE} =~ m/REG/") {next;}
                unless ($lsof->file_attrib(PROC=>$pk, FILE=>$op_f, ATTRIB=>"TYPE") =~ m/REG/) {next;}
##              if ($ps->{FILES}->{$op_f}->{NAME} =~ m/\)$/) {next;}
                if ($lsof->file_attrib(PROC=>$pk, FILE=>$op_f, ATTRIB=>"NAME") =~ m/\/dev\//) {next;}
                print "\n     " . $lsof->file_attrib(PROC=>$pk, FILE=>$op_f, ATTRIB=>"NAME");
              }

            print "\n signature files: ";
            foreach my $sig_f (@{$sig->{OFILES}}) {
                print "\n     $sig_f";
              }
          }

        #
        # -- compute symmetric difference of our two lists (see Cookbook) :
        #

        my %count = ();

        # -- increment count of each regular, non-device, file :
        #
##      foreach my $e (keys %{$ps->{FILES}}) {
        foreach my $e (keys %{$lsof->open_files(PROC=>$pk)}) {

##          unless ($ps->{FILES}->{$e}->{TYPE} =~ m/REG/) {  next;  }
            unless ($lsof->file_attrib(PROC=>$pk, FILE=>$e, ATTRIB=>"TYPE") =~ m/REG/) {next;}
                # ...on lsof v??? on Solaris 7 reports "VREG", while lsof v???
                #    on Linux reports "REG"...

##          if ($ps->{FILES}->{$e}->{NAME} =~ m/\)$/) {next;}
##          if ($ps->{FILES}->{$e}->{NAME} =~ m/\/dev\//) {  next;  }
            if ($lsof->file_attrib(PROC=>$pk, FILE=>$e, ATTRIB=>"NAME") =~ m/\/dev\//) {next;}
                # ...regular files ([V]REG) only;  not devices...

##          $count{$ps->{FILES}->{$e}->{NAME}}++;
            $count{$lsof->file_attrib(PROC=>$pk, FILE=>$e, ATTRIB=>"NAME")}++;
          }

        # -- increment count of expected such files :
        #
        foreach my $e (@{$sig->{OFILES}}) {
            $count{$e}++;
          }

        my @diff = ();
        foreach my $e (keys %count) {
            # -- if counted once, then we have a difference :
            if ($count{$e} == 1) { 
                # -- unless its a "?" open file :
                unless (defined $sig->{QFILES}->{$e}) {
                    push(@diff, $e);
                  }
              }
          }

        # -- log any found differences :
        #
	if (scalar @diff > 0) {
            push(@$detail, "Open-files signature does not match: $cmd" 
                             . sprintf("%10s", $ppid) . sprintf("%10s", $gid));
            foreach (@diff) {  push(@$detail, "    open file sig diff: $_");  }
	    $result = 0;
          }
        else {  $result = 1;  }
      }

    # -- there is no open-files signature :
    #
    else {  $result = 1;  }
                # ...don't define sigs for all things and if not defined assume
                #    we don't want to bother checking...

    return $result;
  }


# -----------------------------------------------------------------------------
# -- IDM_open_files :
# -----------------------------------------------------------------------------


sub _get_sigs_eofs() {
    my $self      = shift;
    my $module    = shift;
    my $mode      = shift;
    my $sigs      = shift;
    my $sigs_file = $self->{SIDSCONFIG}->root_dir() 
                  . "/etc/$module/expected." . lc($mode);

    open(SIGS, $sigs_file) || die "\n\n Can't open $sigs_file \n\n";

    my $dir = "";

    while (my $line = <SIGS>) {
        unless ($line =~ m/\S/) {next}
        chomp($line);
        if ($line =~ m/\s*\#/) {next}
        
        elsif ($line =~ m/^\s*DIR\:\s+\"\/[^\"]+\"\s*$/) {
            $dir = $line;
            $dir =~ s/\s*DIR\:\s*\"(\/[^\"]+)\"\s*/$1/;
            $sigs->{$dir} = "";
          }
        elsif ($line =~ m/^\s*[a-zA-Z0-9\\\/\*\.\-\_\+\[\]]+\s*$/) {
            $line =~ s/\s*//g;
            $sigs->{$dir} .= "\n    |$line";
          }
        else {
            die "\n\n Unknown line: $line \n\n";
          }
      }
    close(SIGS);

    foreach (keys %{$sigs}) {

        $sigs->{$_} =~ s/^\s*\|//;

        $self->debug("\n\n IDM_Sigs::_get_sigs_eof :  DIR: $_, reg-exp :  ");
        $self->debug("\n   " . $sigs->{$_});
      }
  }


# -----------------------------------------------------------------------------
# -- 

sub _matches_eof_sigs() {
    my $self = shift;
    my $of   = shift;
    my $sigs = shift;

    foreach my $dir (keys %$sigs) {


        # -- if the given open file (which includes full path) is in our
        #    current dir :
        #
        if ($of =~ m/^$dir/) {

            # -- remove "$dir" from beginning of open file :
            $of =~ s/^$dir\///;

            # -- sneaky, clever use of a "/x" reg-exp to avoid an 
            #    extra foreach-loop :
            #
            my $string = '$of =~ m/' . $sigs->{$dir} . '/x'; 
            my $result = eval($string);
                # ...m// returns 1 for true or "" for false...

            $self->debug_verbose("\n\nIDM_Sigs::_matches_eof_sigs : open file: //$of//");
            $self->debug_verbose("\n\nIDM_Sigs::_matches_eof_sigs : eval string  {{$string}}");
            $self->debug_verbose("\n\nIDM_Sigs::_matches_eof_sigs : eval error  //$@//");
            $self->debug_verbose("\nIDM_Sigs::_matches_eof_sigs : eval return //$result//");

            return (1, $result);
          }
      }

    return (0, undef);
  }



# -----------------------------------------------------------------------------
# -- IDM_net_conns :
# -----------------------------------------------------------------------------

sub _get_sigs_net() {
    my $self   = shift;
    my $module = shift;
    my $mode   = shift;
    my $sigs   = shift;
    my $sigs_file = "";
    
    $sigs_file = $self->{SIDSCONFIG}->root_dir() 
               . "/etc/$module/expected." . lc($mode);


    open(SIGS, $sigs_file) || die "\n\n Can't open SIGS file: $sigs_file \n\n";

    while (my $line = <SIGS>) {
        unless ($line =~ m/\S/)    {next}
        if     ($line =~ m/\s*\#/) {next}
        unless ($line =~ m/\s*[A-Z]+:\{.+\}\s*(\*\*\*\s*[A-Z]+:\{.+\}\s*)+/) {
            die "\n\n Unrecognised line: $line \n\n";
          }

        my $sig = { };
        my @sig = split(/\*\*\*/, $line);

        # -- ($sig->{COUNTER}, $sig->{CODE}, $sig->{RETURN}) = ...
        foreach my $s (@sig) {
            my ($key, $val) = split(/\:\{/, $s);
            $key =~ s/\s*//g;
            $val =~ s/\}\s*$//;
            $sig->{$key} = $val;
          }

        push(@$sigs, $sig);
      }
    close(SIGS);
  }


# -----------------------------------------------------------------------------
# -- 

sub _matches_net_sigs() {
    my $self = shift;
    my $c    = shift;
    my $sigs = shift;

    foreach my $s (@$sigs) {
        my $string = 'if(' . $s->{CODE} . '){return "' . $s->{RETURN} . '"}';
        my $result = eval $string;

        if (($result eq "expected") || ($result eq "bad_stuff")) {
            return ($result, $s)
          }
      }

    return ("not_found", undef); 
  }



# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------
