# -----------------------------------------------------------------------------
package IDM_cheesewire;

# -- Roughly, a clone of Tripwire;

# -----------------------------------------------------------------------------

use strict;
use vars qw(@ISA);
use IDM_Template;
use IDM_State;
use Dir_MD5;

@ISA = ("IDM_Template", "IDM_State");


# -----------------------------------------------------------------------------
# -- 

sub new() {
    my $class    = shift;
    my $config   = shift;
    my $modsconf = shift;
    my $self     = { };

    bless($self, $class);

    $self->new_template(CLASS      => $class, 
                        SIDSCONFIG => $config, 
                        MODSCONF   => $modsconf);

    $self->{DB}  = $self->_get_cheesewire_db($self->{SIDSCONFIG}->root_dir() 
                                           . "/etc/IDM_cheesewire/db.txt");
    $self->{DB2} = $self->_get_cheesewire_db($self->{SIDSCONFIG}->root_dir() 
                                           . "/etc/IDM_cheesewire/db_2.txt");


    $self->init_state(FIELD      => "COUNTER", 
                      FIELDVALUE => 1,
                      STATEMODE  => "DAEMON");


    if ($self->debug()) {
        $self->debug("\n\n CHEESEWIRE DB :");

        foreach my $dir (keys %{$self->{DB}}) {
            $self->debug(" $dir \t : ");
            foreach my $file (keys %{$self->{DB}->{$dir}}) { 
                $self->debug("     $file \t : ");
                foreach (keys %{$self->{DB}->{$dir}->{$file}}) {
                    $self->debug($_ . " " . $self->{DB}->{$dir}->{$file}->{$_} . " ");
                  }
              }
          } 
        $self->debug("\n\n");
      }

    if ($self->debug_verbose()) {
        $self->debug_verbose("\n\n CHEESEWIRE DB2 :  \n");
        #   
        #
        #
        #
      }

    return $self;
  } 

# -----------------------------------------------------------------------------
# -- Do our thing (this be called each time "around") :

sub run() {
    my $self = shift;
    my $sids = shift;
    my $log  = $self->{LOG};

    $self->run_template($sids);

    $log->append("Gonna run Cheesewire-1 at " . $sids->date_time() . "...");
    $log->indent();
    $self->_run("DB");
    $log->un_indent();
    $log->append();

    if ($self->get_state(FIELD => "COUNTER") == 10) {
        $log->append("Gonna run Cheesewire-2 at " . $sids->date_time() . "...");
        $log->indent();
        $self->_run("DB2");
        $log->un_indent();
        $log->append();

        $self->set_state(FIELD => "COUNTER", FIELDVALUE => 1);
    }

    $self->set_state(FIELD      => "COUNTER", 
                     FIELDVALUE => $self->get_state(FIELD => "COUNTER") + 1);
  }


# -----------------------------------------------------------------------------
# -- PRIVATE TOOLKIT :
# -----------------------------------------------------------------------------

# --

sub _run() {
    my $self    = shift;
    my $db      = shift;
    my $log     = $self->{LOG};
    my $current = { };

    # ...get current MD5s to compare to reference ones...
    foreach my $dir (keys %{$self->{$db}}) {
        print "\n getting current for dir: $dir...";
        $current->{$dir} = Dir_MD5->new(DIR => $dir, TYPE => "SIMPLE");
        print "\n     ...got";
      }

    # ...debug...

    if ($self->debug()) {
        print "\n\n Current vals :";

        foreach my $dir (keys %{$self->{$db}}) {
            foreach my $f (keys %{$self->{$db}->{$dir}}) {
                print "\n $dir/$f " . 
                $current->{$dir}->file_chk_str("$dir/$f");
              }
          }
        print "\n";
      }

    # ...compare...

    $log->append("Comparing to stored values...");
    $log->indent();
    foreach my $dir (keys %{$self->{$db}}) {
        foreach my $f (keys %{$self->{$db}->{$dir}}) {
            my $stats = $current->{$dir}->get_file_stats($dir, $f);
            foreach my $a (keys %{$self->{$db}->{$dir}->{$f}}) {
                unless ($self->{$db}->{$dir}->{$f}->{$a} eq $stats->{$a}) {
		    $log->append("DIFF $dir/$f : ");
                    $log->indent();
                    $log->append("db : " . $a . " " 
                                         . $self->{$db}->{$dir}->{$f}->{$a});
                    $log->append("cu : " . $a . " " . $stats->{$a});
                    $log->un_indent();
                  }
              }
          }
      }

    $log->un_indent();
    $log->append("...compared");
  }


# -----------------------------------------------------------------------------
# --

sub _get_cheesewire_db() {
    my $self = shift;
    my $file = shift;
    my $db   = { };

    open(DB, $file)
        || die "\n\n IDM_cheeswire::_get_cheesewire_db: " . 
               "can't open db file \n\n";

    my $dir = "";

    while(my $line = <DB>) {

        # ...comment...
        if (($line =~ m/^\#/) || ($line =~ m/^\s+/)) { next; }

        # ...date/time stamp...
        if ($line =~ m/^DATETIME\s+/) { next; }

        # ...new directory...
        if ($line =~ m/^DIR\s+/) {
            (undef, $dir) = split(/\s+/, $line);
          } 
        else {
            my ($file, @attribs) = split(/\s+/, $line);

            foreach my $a (@attribs) {
                my ($key, $val) = split(/:/, $a);
                $db->{$dir}->{$file}->{$key} = $val;
              }
          }          
      }

    close(DB);

    return $db;
  }

# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------
