# -----------------------------------------------------------------------------
package IDM_log_monitoring;
# -----------------------------------------------------------------------------

use strict;
use vars qw(@ISA);
use IDM_Template;
use Time::Local;
use File::stat;

####use Algorithm::Diff;
#    ...now using custom-built diff as the above seems to get easily confused
#       and cascade "errors"...

@ISA = ("IDM_Template", "IDM_Common");


# -----------------------------------------------------------------------------
# -- we don't want to warn about any one "missing" line more than once so we
#    keep a record of those we've seen before (for $time_0, at least --- see 
#    subroutine "_diff"...

                                         #### TODO:  IDM_State-ise this
                                         #### TODO:  IDM_State-ise this

my $in_syslog_not_in_tailed_copy = { };
my $in_tailed_copy_not_in_syslog = { };

# ...this can go wrong if their are many differences on startup (and hence 
#    these hashes have not been "initialised" :

my $first_time_around = 1;


# -----------------------------------------------------------------------------
# -- 

sub new() {
    my $class    = shift;
    my $config   = shift;    
    my $modsconf = shift;
    my $self     = { };

    bless($self, $class);

    $self->new_template(CLASS      => $class, 
                        SIDSCONFIG => $config,
                        MODSCONF   => $modsconf);

    # -- override the LOG set by new_template --- this module does not log 
    #    to syslog as might get a feedback situation if something goes wrong...
    #
    $self->{LOG} = Log->new(PREFIX    => $self->{CLASS}, 
                            MODULE    => $self->{CLASS},
                            MODSCFG   => $self->{MODSCONF},
                            SYSLOG    => "no",
                            BUFFERED  => "no",
                            ROOT_DIR  => $config->root_dir(),
                            OWNER_UID => $config->log_owner_uid(),
                            OWNER_GID => $config->log_owner_gid);

    return $self;
  }


# -----------------------------------------------------------------------------
# -- MAIN :
# -----------------------------------------------------------------------------


# -- each time run is called we compare the system log file to our local
#    copy;  we look for missing stuff :

sub run () {
    my $self = shift;
    my $sids = shift;
    my $log  = $self->{LOG};

    $self->run_template($sids);

    foreach (@{$self->{MODSCONF}->mss(CLASS => $self->{CLASS}, CFG => "logs")}) {
        $self->_diff($_);
      }

    $first_time_around = 0;

    $log->append("");
    $log->append("Size of \"in_syslog_not_in_tailed_copy \" :  " . keys %$in_syslog_not_in_tailed_copy);
    $log->append("Size of \"in_tailed_copy_not_in_syslog\" :  " . keys %$in_tailed_copy_not_in_syslog);
  }


# -----------------------------------------------------------------------------
# -- Use our own diff algorithm to compare our local copy of the system log 
#    file in <sids_root>/var/log/IDM_log_monitoring/ to the system copy in
#    (/var/adm or) /var/log --- Algorithm::Diff can cascade "differences" 
#    spectacularly wrongly on occasion for our comparison :

sub _diff() {
    my $self  = shift;
    my $files = shift;
    my $log   = $self->{LOG};
    my $modsc = $self->{MODSCONF};

    my $time_0 = 3600 + 1.1 * $modsc->module_period($self->{CLASS});
        # ...we get lines back as far as 1.1 * module-period plus 1 hour...


    # -- write old lines (> time_0) from $in_syslog_not_in_tailed_copy and
    #    $in_tailed_copy_not_in_syslog out to log file of some kind else
    #    will grow forever :
    #
#    ... 
#    ... 
                                          #### TODO
                                          #### TODO

    $log->append_echo("");
    $log->append_echo("Getting both tailed and system logs...");

    # -- get AT LEAST required lines from BOTH sets of log files (messages, 
    #    messages.0... and local copy) --- the configured list of tail and 
    #    system files (in Modules_Config) should be sufficient, allowing for
    #    log-rotation frequency :

    my $tailed_lines = $self->_get_lines($time_0, $files->{tail_files}, 
                                                  $files->{tail_dir});  
    my $system_lines = $self->_get_lines($time_0, $files->{syst_files},
                                                  $files->{syst_dir});


    if ($self->debug_verbose()) {
        $self->log_bounding_lines($system_lines,  $tailed_lines, 
                                  "syslog lines", "tailed lines");
      }

    # -- trim so have same number of lines :
   
    if ($self->debug_verbose()) {
        $log->append("");
        $log->append("Trimming to same length...");
      }

    while (scalar @$tailed_lines > scalar @$system_lines) {  shift @$tailed_lines;  }
    while (scalar @$system_lines > scalar @$tailed_lines) {  shift @$system_lines;  }

    if ($self->debug_verbose()) {
        $self->log_bounding_lines($system_lines,  $tailed_lines, 
                                  "syslog lines", "tailed lines");
      }

#    print "\n Determining diff of tailed and system logs...";
#    my $diffs = &Algorithm::Diff::diff($tailed_lines, $system_lines);
#
#    print "\n\n DIFFS : " . @$diffs . "\n\n";

    my %diff_s = ();
    my %diff_t = ();

    foreach my $s (@$system_lines) { $diff_s{$s} = "S"; }
    foreach my $t (@$tailed_lines) { $diff_t{$t} = "T"; }

    $log->append("");
    $log->append("In /var syslogs but not tailed copy:");
    $log->indent();

    my $c = 0;

    foreach my $s (sort keys %diff_s) {

        # -- unless its in tailed-copy too --- or seen as a diff line already :
        #
        unless (    (defined $diff_t{$s}) 
                 || (defined $in_syslog_not_in_tailed_copy->{$s})) {  
            my $tmp = $s;  chomp($tmp);  $log->append($tmp);
            $c++;
            $in_syslog_not_in_tailed_copy->{$s} = 1;
          }

        # -- unless we've just started, if there are too many different lines, 
        #    skip and warn :
        #
        unless ($first_time_around) {
            if ($c > $modsc->mss(CLASS => $self->{CLASS}, CFG => "mdle")) {
                $log->append_echo_warn("Too many entries in /var syslogs but not in " 
                                     . "tailed copy, skipping... Is tail running?");
                last;
              }
          }
      }

    $log->un_indent();
    $log->append("...end..");
    $log->append("");
    $log->append("In tailed copy, but not /var syslogs:");
    $log->indent();

    $c = 0;

    foreach my $t (sort keys %diff_t) {

        # -- unless its in var-syslogs too  --- or seen as a diff line already :
        #
        unless (    (defined $diff_s{$t}) 
                 || (defined $in_tailed_copy_not_in_syslog->{$t})) {  
            my $tmp = $t;  chomp($tmp);  $log->append($tmp);
            $c++;
            $in_tailed_copy_not_in_syslog->{$t} = 1;
          }

        # -- if there are many different lines, skip and warn :
        #
        unless ($first_time_around) {
            if ($c > $modsc->mss(CLASS => $self->{CLASS}, CFG => "mdle")) {
                $log->append_echo_warn("Too many entries in tailed copy but not in "
                                     . "/var syslogs, skipping... Is tail running?");
                last;
              }
          }
      }

    $log->un_indent();
    $log->append("...end..");
  }


# -----------------------------------------------------------------------------
# -- open each log file in given list, grab lines and catenate them all 
#    together, until we have sufficient --- i.e., the date-time stamp 
#    difference between first and last is greater than our module-period :

sub _get_lines() {
    my $self   = shift;
    my $time_0 = shift;
    my $files  = shift;
    my $path   = shift;
    my $log    = $self->{LOG};

    my @ls = ();

    # -- foreach log file in given list, grab lines and catenate them all 
    #    together into one long list :
    # 
    foreach my $lfs (@$files) {
 
        my @l  = ();
        my @t  = ();
        my $o_file = "$path/$lfs";

        open(TL, $o_file) 
            || die "\n\n IDM_log_mon::get_tailed_lines :  " 
                 . "Can't open file: $o_file\n\n";
        @t = <TL>;
        close(TL);

        # -- filter out any shit --- does line begin with Mar 19 16:31:35, or 
        #    similar? :
        foreach my $t (@t) {
            if ($t =~ m/^\s*[A-Z][a-z]+\s+[0-9]+\s+[0-9]+\:[0-9]+\:[0-9]+/) {
                push(@l, $t);
              }
          }

        @ls = (@l, @ls);
            # ...stick lines from current (older) file at beginning...

        $log->append_echo("");
        $log->append_echo("Just read file: $o_file");
        if ($self->debug()) {
            $self->log_bounding_lines(\@l,  \@ls, "just-read lines", "total lines");
	  }

        # -- if we've not read an empty file (as we might after a taillog-rotation)
        #    and the time diff between our current first and last lines is sufficient :
        #
        if (    (defined $ls[0])  && ($ls[0] =~ m/\S/) 
             && (  $self->_syslog_lines_dt_diff($ls[0], $ls[scalar(@ls) - 1]) 
                 > $time_0)) {

            $log->append_echo("");
            $log->append_echo("All required log files found.");
            return \@ls;
          }
      }
    
    $log->append_echo("");
    $log->append_echo_warn("NOT all required log files found");
    return \@ls;
  }


# -----------------------------------------------------------------------------
# -- determine the difference in date-time stamp of two syslog lines (taking 
#    into account the possible case of a year-change between them) :

my %mon_num = ("Jan" => 0,      "Feb" => 1,      "Mar" => 2, 
               "Apr" => 3,      "May" => 4,      "Jun" => 5,            
               "Jul" => 6,      "Aug" => 7,      "Sep" => 8, 
               "Oct" => 9,      "Nov" => 10,     "Dec" => 11);


sub _syslog_lines_dt_diff() {
    my $self   = shift;
    my $line_0 = shift;
    my $line_n = shift;
    my $log    = $self->{LOG};

    my ($month_0, $day_0, $time_0)     = split(/\s+/, $line_0);
    my ($hour_0, $minute_0, $second_0) = split(/:/, $time_0);

    my ($month_n, $day_n, $time_n)     = split(/\s+/, $line_n);
    my ($hour_n, $minute_n, $second_n) = split(/:/, $time_n);

    my $mon_num_0 = "fred";
    my $mon_num_n = "fred";

    if ((defined $mon_num{$month_0}) && (defined $mon_num{$month_n})) {
        $mon_num_0 = $mon_num{$month_0};
        $mon_num_n = $mon_num{$month_n};
      }
    else {
        $log->append_echo_warn("Unknown month [$month_0, $month_n] in line: \n "
            . $line_0 . "\n and/or \n" . $line_n);
        return 1;
      } 

    unless (($day_0 =~ m/\S/) && ($day_n =~ m/\S/)) {
        $log->append_echo_warn("Unknown day [$day_0, $day_n] in line: \n" 
            . $line_0 . " \n and/or \n" . $line_n);
        return 1;
      }

    #
    # -- syslogs don't incorporate the year, and we don't care what it
    #    is, as we only want the difference (in seconds) between the dates,
    #    so we must present (to timelocal) either the same year, e.g. 2000,
    #    or $year_n - $year_0 = 1 in the special case where New Year has
    #    occurred within the log :
    #   

    my $year_n = 2000; 
    my $year_0 = 2000;

    if ($mon_num_n < $mon_num_0) {  $year_n++; }

    my $diff_seconds = 
        timelocal($second_n, $minute_n,  $hour_n, 
                  $day_n,    $mon_num_n, $year_n - 1900)
        - timelocal($second_0, $minute_0,  $hour_0, 
                    $day_0,    $mon_num_0, $year_0 - 1900);

    return $diff_seconds;
  }


# -----------------------------------------------------------------------------
# -- log first and last entries of two lists of syslog lines, with some
#    explanatory text :

sub log_bounding_lines() {
    my $self    = shift;
    my $lines_1 = shift;
    my $lines_2 = shift;
    my $str_1   = shift;
    my $str_2   = shift;
    my $log     = $self->{LOG};

    $log->append("");
    $log->append("Read " . scalar @$lines_1 . " " . $str_1);
    $log->append("Read " . scalar @$lines_2 . " " . $str_2);

    my $first = $lines_1->[0];
    my $last  = $lines_1->[scalar(@$lines_1) - 1];

    chomp($first);
    chomp($last);

    $log->append("");
    $log->append("Bounding " . $str_1 . " read:");
    $log->indent();
    $log->append($first);
    $log->append($last);
    $log->un_indent();

    $first = $lines_2->[0];
    $last  = $lines_2->[scalar(@$lines_2) - 1];

    chomp($first);
    chomp($last);

    $log->append("");
    $log->append("Bounding " . $str_2 . " read:");
    $log->indent();
    $log->append($first);
    $log->append($last);
    $log->un_indent();

  }

# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------
