# -----------------------------------------------------------------------------
package IDM_open_files;
# -----------------------------------------------------------------------------

use strict;
use vars qw(@ISA);
use IDM_Sigs;
use IDM_Template;
use Lsof;
use Netstat;

@ISA = ("IDM_Template", "IDM_Sigs");


my $sigs_pform = { };
my $sigs_local = { };


sub new() {
    my $class    = shift;
    my $config   = shift;    
    my $modsconf = shift;    
    my $self     = { };

    bless($self, $class);

    $self->new_template(CLASS      => $class,
                        SIDSCONFIG => $config,
                        MODSCONF   => $modsconf);

    $self->_get_sigs_eofs("IDM_open_files", $config->platform() . ".lsof", $sigs_pform);
    $self->_get_sigs_eofs("IDM_open_files", "local.lsof", $sigs_local);

    return $self;
  }



# -----------------------------------------------------------------------------
# -- 
# -----------------------------------------------------------------------------

sub run () {
    my $self = shift;
    my $sids = shift;

    my $modsconf = $self->{MODSCONF};

    $self->run_template($sids);

    if ($modsconf->mss(CLASS => $self->{CLASS}, CFG => "run_lsof_binary")) {
        $self->run_lsof_binary();
      }
  }


# -----------------------------------------------------------------------------
# --
# -----------------------------------------------------------------------------

sub run_lsof_binary() {

    my $self  = shift;
    my $lsof  = Lsof->new(FLAGS => "+M", LOG => $self->{LOG});
    my $procs = $lsof->procs();

    my @unexpected = ();

    # -- loop through processes :
    #
    foreach my $p (keys %$procs) {

        $self->debug_verbose("\n\n IDM_open_files::run_lsof_binary : PROC  : "
                           . "\n\n $p ::"
                           . "\n\n IDM_open_files::run_lsof_binary : FILES : ");

        # -- loop through each file opened by the process :
        #
        foreach my $f (keys %{$lsof->open_files(PROC => $p)}) {

            my $type = $lsof->file_attrib(PROC=>$p, FILE=>$f, ATTRIB=>"TYPE");

            # -- IP sockets and unix sockets belong to other modules :
            # 
            if ($lsof->is_type_net($type) || $lsof->is_type_unix_socket($type)) {
                next;
              }

            my ($found_l, $matches_l) = $self->_matches_eof_sigs($f, $sigs_local);
            my ($found_p, $matches_p) = $self->_matches_eof_sigs($f, $sigs_pform);

            if ($found_l) {
                unless ($matches_l) {  push(@unexpected, $f);  }
	      }
            elsif ($found_p) {
                unless ($matches_p) {  push(@unexpected, $f);   }
              }
          }
      }


    #
    # -- log results :
    #

    my $log = $self->{LOG};

    $log->output_title("LSOF BINARY");
    $log->output_array("Unexpected", \@unexpected, ['%-32s', '$c']);
  }


# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------
