# -----------------------------------------------------------------------------
package IDM_Common;
# -----------------------------------------------------------------------------

use strict;


# -----------------------------------------------------------------------------
# -- IDM_procs
# -----------------------------------------------------------------------------


sub _get_procs_lsof_sigs() {
    my $self = shift;
    my $mode = shift;
    my $sigs = shift;

    my $root = $self->{SIDSCONFIG}->root_dir() . "/etc/IDM_procs/expected";
    my $file = $root . "." . lc($mode);


    open(CFG, $file) || 
        die "\n\n IDM_procs::_get_procs_lsof_sigs :  Can't open file: $file \n\n";

    my $ps_name = "";
    my $BLOCK   = "";

    while(my $line = <CFG>) {
        chomp($line);

        if     ($line =~ m/^\s*\#/) {next}
        unless ($line =~ m/\S/)     {next}

        if ($line =~ m/^PS\:/){
            $line =~ s/^PS\:\s*//;
            $line =~ s/\s$//;
            $ps_name = $line;
            $sigs->{$ps_name} = { };
            undef $BLOCK;
	  }

        elsif ($line =~ m/\s+CWD|OFILES|ODEVS|INET\s+\=\s*/) {
            $BLOCK = $line;
            $BLOCK =~ s/\s+([A-Z]+)\s+\=\s*/$1/;
            $sigs->{$ps_name}->{$BLOCK} = [ ];
          }

        elsif ($line =~ m/\s+QFILES|QDEVS\s+\=\s*/) {
            $BLOCK = $line;
            $BLOCK =~ s/\s+([A-Z]+)\s+\=\s*/$1/;
            $sigs->{$ps_name}->{$BLOCK} = { };
          }

        elsif ($line =~ m/^\s*\"[^\"]+\"\s*$/) {
            $line =~ s/^\s*//g;
            $line =~ s/\s*$//g;
            $line =~ s/\"//g;         

            if (($BLOCK eq "QFILES") || ($BLOCK eq "QDEVS")) {
                $sigs->{$ps_name}->{$BLOCK}->{$line} = 1;
              }
            elsif (($BLOCK eq "OFILES") || ($BLOCK eq "ODEVS")) {
                push(@{$sigs->{$ps_name}->{$BLOCK}}, $line);
              }
            elsif ($BLOCK eq "CWD") {
                push(@{$sigs->{$ps_name}->{CWD}}, $line);
	      }
            elsif ($BLOCK eq "INET") {
                # ...not done yet...
	      }
            else {
                die "\n\n Unknown block: $BLOCK \n\n";
              }
          }
        else {
            die "\n\n Unknown line: $line \n\n";
          }
      }
    close(CFG);
  }




# -----------------------------------------------------------------------------

sub _get_procs_api_sigs() {
    my $self = shift;
    my $mode = shift;
    my $sigs = shift;

    my $root = $self->{SIDSCONFIG}->root_dir() . "/etc/IDM_procs/expected";
    my $file = $root . "." . lc($mode);

    my @cmds_eq  = ();
    my @cmds_ma  = ();

        open(SIGS, $file) 
            || die "\n\n IDM_procs::_get_procs_api_sigs :  " 
                 . "cannot open file: $file//\n\n";

        while(my $line = <SIGS>) {
            chomp($line);

            unless ($line =~ m/\S/){next;}
            if ($line =~ m/^\s*\#/){next;}
           
            if ($line =~ m/^EQ\s\"[A-Za-z0-9\/\-\_\ \.\:]+\"\s*$/) {
                $line =~ s/EQ\s\"([A-Za-z0-9\/\-\_\ \.\:]+)\"/$1/;
                push(@cmds_eq, $line);
              }
            elsif ($line =~ m/^MA\s\"[A-Za-z0-9\/\-\_\ \.\:]+\"\s*$/) {
                $line =~ s/MA\s\"([A-Za-z0-9\/\-\_\ \.\:]+)\"/$1/;
                push(@cmds_ma, $line);
              }
            else{
                die "\n\n IDM_procs::_get_procs_api_sigs :  " 
                  . "unknown line: $line//\n\n";
              }
          }
        close(SIGS);

    $sigs->{EQ} = \@cmds_eq;
    $sigs->{MA} = \@cmds_ma;

    return $sigs;
  }


# -----------------------------------------------------------------------------
# --


sub _matches_proc_sigs() {
    my $self = shift;
    my $proc = shift;
    my $sigs = shift;

    foreach my $s (@{$sigs->{EQ}}) {
         if ($proc->cmndline() eq $s) {return 1}
      }

    foreach my $s (@{$self->{MA}}) {
        if ($proc->cmndline() =~ m/$s/) {return 1}
      }

    return 0;
  }



# -----------------------------------------------------------------------------
# -- IDM_open_files :
# -----------------------------------------------------------------------------


sub _get_sigs_eofs() {
    my $self      = shift;
    my $module    = shift;
    my $mode      = shift;
    my $sigs      = shift;
    my $sigs_file = $self->{SIDSCONFIG}->root_dir() 
                  . "/etc/$module/expected." . lc($mode);

    open(SIGS, $sigs_file) || die "\n\n Can't open $sigs_file \n\n";

    my $dir = "";

    while (my $line = <SIGS>) {
        unless ($line =~ m/\S/) {next}
        chomp($line);
        if ($line =~ m/\s*\#/) {next}
        
        elsif ($line =~ m/^\s*DIR\:\s+\"\/[^\"]+\"\s*$/) {
            $dir = $line;
            $dir =~ s/\s*DIR\:\s*\"(\/[^\"]+)\"\s*/$1/;
            $sigs->{$dir} = "";
          }
        elsif ($line =~ m/^\s*[^\/]+\s*$/) {
            $line =~ s/\s*//g;
            $sigs->{$dir} .= "\n    |$line";
          }
        else {
            die "\n\n Unknown line: $line \n\n";
          }
      }
    close(SIGS);

    foreach (keys %{$sigs}) {
        $sigs->{$_} =~ s/^\s*\|//;
        print "\nX:" . $sigs->{$_};
      }

  }



# -----------------------------------------------------------------------------
# -- IDM_net_conns :
# -----------------------------------------------------------------------------

sub _get_sigs_net() {
    my $self   = shift;
    my $module = shift;
    my $mode   = shift;
    my $sigs   = shift;
    my $sigs_file = "";
    
    $sigs_file = $self->{SIDSCONFIG}->root_dir() 
               . "/etc/$module/expected." . lc($mode);


    open(SIGS, $sigs_file) || die "\n\n Can't open SIGS file: $sigs_file \n\n";

    while (my $line = <SIGS>) {
        unless ($line =~ m/\S/)    {next}
        if     ($line =~ m/\s*\#/) {next}
        unless ($line =~ m/\s*[A-Z]+:\{.+\}\s*(\*\*\*\s*[A-Z]+:\{.+\}\s*)+/) {
            die "\n\n Unrecognised line: $line \n\n";
          }

        my $sig = { };
        my @sig = split(/\*\*\*/, $line);

        # -- ($sig->{COUNTER}, $sig->{CODE}, $sig->{RETURN}) = ...
        foreach my $s (@sig) {
            my ($key, $val) = split(/\:\{/, $s);
            $key =~ s/\s*//g;
            $val =~ s/\}\s*$//;
            $sig->{$key} = $val;
          }

        push(@$sigs, $sig);
      }
    close(SIGS);
  }


# -----------------------------------------------------------------------------
# -- 

sub _matches_net_sigs() {
    my $self = shift;
    my $c    = shift;
    my $sigs = shift;

    foreach my $s (@$sigs) {
        my $string = 'if(' . $s->{CODE} . '){return "' . $s->{RETURN} . '"}';
        my $result = eval $string;

        if (($result eq "expected") || ($result eq "bad_stuff")) {
            return ($result, $s)
          }
      }

    return ("not_found", undef); 
  }



# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------
