# -----------------------------------------------------------------------------
package Modules_Config;
# -----------------------------------------------------------------------------


use strict;


sub new () {
    my $class = shift;
    my $self  = { };
    my $self  = { DEBUGMODE => "standard_run",
                  @_ };
        # ...choose a debug mode: "debug", "debug_verbose" or 
        #    "standard_run" (the latter means no debug info)...

    bless($self, $class);

    # -- default mode for modules :
    #
    if(    ($self->{DEBUGMODE} eq "debug") 
        || ($self->{DEBUGMODE} eq "debug_verbose") 
        || ($self->{DEBUGMODE} eq "standard_run")) {
      }
    else {
        die "\n\n  *** DEBUGMODE???  (" . $self->{DEBUGMODE} . ")\n\n";
      }


    #
    # -- modules to use :
    #

        # ...put IDM_log_monitoring first else other modules may squirt stuff
        #    at system logs and the tail-script not have a chance to catch
        #    up so false stuff gets reported...

    $self->{MODS_TO_USE} = {
#                            "IDM_cheesewire"     => 1,
#                            "IDM_log_monitoring" => 1,
#                            "IDM_net_conns"      => 1,
                             "IDM_procs"          => 1,
#                            "IDM_open_files"     => 1
                            };

#                            "IDM_log_monitoring",
#                            "IDM_cheesewire",
#                            "IDM_ps_auxww_root",
#                            "IDM_net_conns", "IDM_proc",
#                            "IDM_net_conns",
#                            "IDM_proc",
#                            "IDM_lsof",
#                            "IDM_net_conns", "IDM_lsof",


     #
     # -- use default (above) if not mentioned here :
     #

     $self->{MODULE_DEBUGMODES} = { "IDM_net_conns"  => "standard_run",
                               "IDM_cheesewire" => "standard_run" };

    #
    # -- minimum times for each module :
    #

     $self->{MODULE_PERIODS} = { "IDM_net_conns"      => 15, 
                                 "IDM_log_monitoring" => 15, 
                                 "IDM_procs"          => 30, 
                                 "IDM_lsof"           => 45, 
                                 "IDM_open_files"     => 15, 
                                 "IDM_cheesewire"     => 60 };

     $self->{MODULE_ROT_PRDS} = { "IDM_net_conns"      => 300, 
                                  "IDM_log_monitoring" => 300, 
                                  "IDM_procs"          => 600, 
                                  "IDM_lsof"           => 900, 
                                  "IDM_open_files"     => 300, 
                                  "IDM_cheesewire"     => 1200 };

#    $self->{MODULE_PERIODS} = { "IDM_net_conns"      => 300, 
#                                "IDM_procs"          => 300, 
#                                "IDM_lsof"           => 600, 
#                                "IDM_open_files"     => 600, 
#                                "IDM_log_monitoring" => 300, 
#                                "IDM_cheesewire"     => 3600 };

#     $self->{MODULE_ROT_PRDS} = { "IDM_net_conns"      => 6000, 
#                                  "IDM_log_monitoring" => 6000, 
#                                  "IDM_procs"          => 12000, 
#                                  "IDM_lsof"           => 18000, 
#                                  "IDM_open_files"     => 6000, 
#                                  "IDM_cheesewire"     => 36000 };

    return $self;
  }


# -----------------------------------------------------------------------------
# -- WHICH MODULES ?
# -----------------------------------------------------------------------------


sub modules_to_use() {
    my $self    = shift;
    my @modules = keys %{$self->{MODS_TO_USE}};

    return \@modules;
  }


# -----------------------------------------------------------------------------
# -- 

sub module_debugmode() {
    my $self   = shift;
    my $module = shift;

    unless (defined $self->{MODS_TO_USE}->{$module}) {
        die "\n\n Modules_Config::module_mode : unknown module : $module \n\n";
      }

    if (defined $self->{MODULE_DEBUGMODES}->{$module}) {
        return $self->{MODULE_DEBUGMODES}->{$module};
      }
    else {
        return $self->{DEBUGMODE};
      }
  }



# -----------------------------------------------------------------------------
# -- TIMES :
# -----------------------------------------------------------------------------

sub module_period() {
    my $self        = shift;
    my $module_name = shift;

    if (defined $self->{MODULE_PERIODS}->{$module_name}) {
        return $self->{MODULE_PERIODS}->{$module_name};
      }
    else {
        die "\n\n SID_Config::module_period :  " 
          . "period undef for module: $module_name. \n\n";
      }
  }


sub module_rot_prd() {
    my $self        = shift;
    my $module_name = shift;

    if (defined $self->{MODULE_ROT_PRDS}->{$module_name}) {
        return $self->{MODULE_ROT_PRDS}->{$module_name};
      }
    else {
        die "\n\n SID_Config::module_rot_prd :  " 
          . "rotation period undef for module: $module_name. \n\n";
      }
  }




# -----------------------------------------------------------------------------
# -- MODULE-SPECIFIC STUFF :
# -----------------------------------------------------------------------------


my @logs     = ();

my $auth     = { };
my $iptables = { };
my $messages = { };

## $messages->{syst_dir} = "/var/adm";
## $messages->{tail_dir} = "/home/mpciish2/src/_sids/var/log";

$messages->{name}     = "messages";
$messages->{syst_dir} = "/var/log";
$messages->{tail_dir} = "/users/simonh/src/_sids/var/log/IDM_log_monitoring";

## -- Solaris likes .0 endings...
##
## $messages->{syst_files} = ["messages", "messages.0", "messages.1"];
## $messages->{tail_files} = ["taillog_messages", "taillog_messages.0", 
##                                                "taillog_messages.1"];
##
## ...Linux does not do .0 endings...
##
$messages->{syst_files} = ["messages", "messages.1", "messages.2"];
$messages->{tail_files} = ["messages", "messages.1", "messages.2"];

$iptables->{name}     = "iptables";
$iptables->{syst_dir} = "/var/log";
$iptables->{tail_dir} = "/users/simonh/src/_sids/var/log/IDM_log_monitoring";

$iptables->{syst_files} = ["iptables", "iptables.1", "iptables.2"];
$iptables->{tail_files} = ["iptables", "iptables.1", "iptables.2"];

$auth->{name}     = "auth";
$auth->{syst_dir} = "/var/log";
$auth->{tail_dir} = "/users/simonh/src/_sids/var/log/IDM_log_monitoring";

$auth->{syst_files} = ["auth", "auth.1", "auth.2"];
$auth->{tail_files} = ["auth", "auth.1", "auth.2"];

push(@logs, $auth);
push(@logs, $iptables);
push(@logs, $messages);

my $max_diff_log_entries = 16;


# -----------------------------------------------------------------------------


my %net_conns_cfg  = ();
my %procs_cfg      = ();
my %log_mon_cfg    = ();
my %open_files_cfg = ();


# --- IDM_open_files :

$open_files_cfg{run_lsof_binary} = 1;


# --- IDM_procs :

$procs_cfg{run_api}  = 1;
$procs_cfg{run_lsof} = 1;


# --- IDM_net_conns :

$net_conns_cfg{run_netstat_binary} = 1;
$net_conns_cfg{run_lsof_binary}    = 1;
$net_conns_cfg{run_proc_net_proto} = 0;
$net_conns_cfg{run_api}            = 0;


# --- IDM_log_monitoring :

$log_mon_cfg{logs} = \@logs;
$log_mon_cfg{mdle} = $max_diff_log_entries;


# -----------------------------------------------------------------------------
# -- 

sub mss() {
    my $self = shift;
    my %args = @_;

    unless ((defined $args{CLASS}) && (defined $args{CFG})) {
        die "\n\n Modules_Config::mss :  CLASS???  CFG??? \n\n";
      }

    if (    ($args{CLASS} eq "IDM_net_conns") 
         && (defined $net_conns_cfg{$args{CFG}})) {
        return $net_conns_cfg{$args{CFG}}
      }
    elsif (    ($args{CLASS} eq "IDM_log_monitoring") 
            && (defined $log_mon_cfg{$args{CFG}})) {  
        return $log_mon_cfg{$args{CFG}}
      }
    elsif (    ($args{CLASS} eq "IDM_procs") 
            && (defined $procs_cfg{$args{CFG}})) {  
        return $procs_cfg{$args{CFG}}
      }
    elsif (    ($args{CLASS} eq "IDM_open_files") 
            && (defined $open_files_cfg{$args{CFG}})) {  
        return $open_files_cfg{$args{CFG}}
      }
    else {
        die "\n\n Modules_Config::mss :  CLASS = $args{CLASS} = ??? \n\n";
      }
  }


# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------

